﻿using System;
using System.Collections.Generic;
using System.Configuration;
using System.Linq;
using System.Net;
using System.Net.Mail;
using System.Text;
using System.Text.RegularExpressions;
using System.Threading.Tasks;
using WebChatApi.Common.Interface;

namespace WebChatApi.Common
{
    /// <summary>
    /// 
    /// </summary>
    public class MailSender : IMailSender, IIdentifiable, IChangePassword
    {
        #region Properties

        /// <summary>
        /// Gets or sets the key.
        /// </summary>
        /// <value>
        /// The key.
        /// </value>
        public Guid Key
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the SMTP address.
        /// </summary>
        /// <value>
        /// The SMTP address.
        /// </value>
        public string SmtpAddress
        {
            get;
            protected set;
        }

        /// <summary>
        /// Gets or sets the name of the user.
        /// </summary>
        /// <value>
        /// The name of the user.
        /// </value>
        public string UserName
        {
            get;
            protected set;
        }

        /// <summary>
        /// Gets or sets the full mail address.
        /// </summary>
        /// <value>
        /// The full mail address.
        /// </value>
        public string FullMailAddress
        {
            get;
            protected set;
        }

        /// <summary>
        /// Gets or sets the password.
        /// </summary>
        /// <value>
        /// The password.
        /// </value>
        public string Password
        {
            get;
            set;
        }

        /// <summary>
        /// Gets or sets the sender identity.
        /// </summary>
        /// <value>
        /// The sender identity.
        /// </value>
        public string SenderIdentity
        {
            get;
            protected set;
        }

        #endregion

        #region Constructor

        /// <summary>
        /// Initializes a new instance of the <see cref="MailSender"/> class.
        /// </summary>
        /// <param name="key">The key.</param>
        /// <param name="smtpAddress">The SMTP address.</param>
        /// <param name="userName">Name of the user.</param>
        /// <param name="password">The password.</param>
        /// <param name="fullMailAddress">The full mail address.</param>
        public MailSender(Guid key, string smtpAddress, string userName, string password, string fullMailAddress)
        {
            this.Key = key;
            this.SmtpAddress = smtpAddress;
            this.UserName = userName;
            this.Password = password;
            this.FullMailAddress = fullMailAddress;
        }

        /// <summary>
        /// Initializes a new instance of the <see cref="MailSender"/> class.
        /// </summary>
        /// <param name="smtpAddress">The SMTP address.</param>
        /// <param name="userName">Name of the user.</param>
        /// <param name="password">The password.</param>
        /// <param name="fullMailAddress">The full mail address.</param>
        public MailSender(string smtpAddress, string userName, string password, string fullMailAddress)
            : this(Guid.NewGuid(), smtpAddress, userName, password, fullMailAddress)
        {
        }

        #endregion

        #region Public methods

        /// <summary>
        /// Sends the mail.
        /// </summary>
        /// <param name="mailMessage">The mail message.</param>
        /// <param name="fromDisplay">From display.</param>
        /// <exception cref="OperationFailureException">SendMail</exception>
        public virtual void SendMail(MailMessage mailMessage, string fromDisplay)
        {
            this.CheckNullObject(mailMessage, "mailMessage");

            try
            {
                SmtpClient smtpClient = new SmtpClient();

                smtpClient.Credentials = new NetworkCredential(this.FullMailAddress, this.Password);
                this.SenderIdentity = this.FullMailAddress;
                if (this.SmtpAddress.Contains(':'))
                {
                    string[] splitted = this.SmtpAddress.Split(':');
                    smtpClient.Host = splitted[0];
                    smtpClient.Port = splitted[1].DBToInt32();
                }
                else
                {
                    smtpClient.Host = this.SmtpAddress;
                }
                mailMessage.From = new MailAddress(this.FullMailAddress);
                smtpClient.Send(mailMessage);
            }
            catch (Exception ex)
            {
                throw new OperationFailureException("SendMail", ex, mailMessage);
            }
        }

        /// <summary>
        /// Sends the mail.
        /// </summary>
        /// <param name="subject">The subject.</param>
        /// <param name="body">The body.</param>
        /// <param name="isHtml">if set to <c>true</c> [is HTML].</param>
        /// <param name="encoding">The encoding.</param>
        /// <param name="toList">To list.</param>
        /// <param name="ccList">The cc list.</param>
        /// <param name="bccList">The BCC list.</param>
        /// <param name="attachments">The attachments.</param>
        /// <param name="mailPriority">The mail priority.</param>
        public void SendMail(string subject, string body, bool isHtml, Encoding encoding, MailAddress[] toList, MailAddress[] ccList, MailAddress[] bccList, List<Attachment> attachments, MailPriority mailPriority)
        {
            MailMessage mailMessage = GenerateMailMessage(subject, body, isHtml, encoding, toList, ccList, bccList, attachments, mailPriority);
            SendMail(mailMessage, null);
        }

        #endregion

        #region GenerateMailMessage

        /// <summary>
        /// Generates the mail message.
        /// </summary>
        /// <param name="subject">The subject.</param>
        /// <param name="body">The body.</param>
        /// <param name="isHtml">if set to <c>true</c> [is HTML].</param>
        /// <param name="encoding">The encoding.</param>
        /// <param name="toList">To list.</param>
        /// <param name="ccList">The cc list.</param>
        /// <param name="bccList">The BCC list.</param>
        /// <param name="attachments">The attachments.</param>
        /// <param name="mailPriority">The mail priority.</param>
        /// <returns></returns>
        /// <exception cref="InvalidObjectException">toList</exception>
        public static MailMessage GenerateMailMessage(string subject, string body, bool isHtml, Encoding encoding, MailAddress[] toList, MailAddress[] ccList, MailAddress[] bccList, List<Attachment> attachments, MailPriority mailPriority = MailPriority.Normal)
        {
            MailMessage mailMessage = new MailMessage();

            mailMessage.CheckNullObject(toList, "toList");
            if (toList.Length == 0)
            {
                throw new InvalidObjectException("toList");
            }

            mailMessage.Subject = mailMessage.GetStringValue(subject);
            mailMessage.Body = mailMessage.GetStringValue(body);
            mailMessage.BodyEncoding = encoding == null ? Encoding.UTF8 : encoding;
            mailMessage.IsBodyHtml = isHtml;

            foreach (MailAddress to in toList)
            {
                mailMessage.To.Add(to);
            }

            if (ccList != null && ccList.Length > 0)
            {
                foreach (MailAddress cc in ccList)
                {
                    if (cc != null)
                        mailMessage.CC.Add(cc);
                }
            }

            if (bccList != null && bccList.Length > 0)
            {
                foreach (MailAddress bcc in bccList)
                {
                    if (bcc != null)
                        mailMessage.Bcc.Add(bcc);
                }
            }

            if (attachments != null && attachments.Count > 0)
            {
                foreach (var attachment in attachments)
                {
                    mailMessage.Attachments.Add(attachment);
                }
            }

            mailMessage.Priority = mailPriority;

            return mailMessage;
        }


        /// <summary>
        /// Generates the mail message.
        /// </summary>
        /// <param name="subject">The subject.</param>
        /// <param name="body">The body.</param>
        /// <param name="toList">To list.</param>
        /// <param name="ccList">The cc list.</param>
        /// <param name="bccList">The BCC list.</param>
        /// <param name="attachments">The attachments.</param>
        /// <param name="mailPriority">The mail priority.</param>
        /// <returns></returns>
        public static MailMessage GenerateMailMessage(string subject, string body, MailAddress[] toList, MailAddress[] ccList, MailAddress[] bccList, List<Attachment> attachments, MailPriority mailPriority)
        {
            return GenerateMailMessage(subject, body, true, Encoding.UTF8, toList, ccList, bccList, attachments, mailPriority);
        }

        /// <summary>
        /// Generates the mail message.
        /// </summary>
        /// <param name="subject">The subject.</param>
        /// <param name="body">The body.</param>
        /// <param name="isHtml">if set to <c>true</c> [is HTML].</param>
        /// <param name="encoding">The encoding.</param>
        /// <param name="toList">To list.</param>
        /// <param name="ccList">The cc list.</param>
        /// <returns></returns>
        public static MailMessage GenerateMailMessage(string subject, string body, bool isHtml, Encoding encoding, MailAddress[] toList, MailAddress[] ccList = null)
        {
            return GenerateMailMessage(subject, body, isHtml, encoding, toList, ccList, null, null, MailPriority.Normal);
        }

        /// <summary>
        /// Generates the mail message.
        /// </summary>
        /// <param name="subject">The subject.</param>
        /// <param name="body">The body.</param>
        /// <param name="toList">To list.</param>
        /// <param name="ccList">The cc list.</param>
        /// <returns></returns>
        public static MailMessage GenerateMailMessage(string subject, string body, MailAddress[] toList, MailAddress[] ccList = null)
        {
            return GenerateMailMessage(subject, body, true, Encoding.UTF8, toList, ccList, null, null, MailPriority.Normal);
        }

        /// <summary>
        /// Generates the mail message.
        /// </summary>
        /// <param name="mailToString">The mail to string.
        /// <example> <![CDATA[ mailto:manish@simplygraphix.com?subject=&body=&cc=anotheremailaddress@anotherdomain.com&bcc=onemore@anotherdomain.com ]]></example>
        /// </param>
        /// <returns></returns>
        /// <exception cref="InvalidObjectException">mailToString;null</exception>
        public static MailMessage GenerateMailMessage(string mailToString)
        {
            const string mailToPrefix = "mailto:";
            MailMessage mailMessage = new MailMessage();
            mailMessage.CheckNullObject(mailToString, "mailToString");

            mailToString = mailToString.Trim();
            if (!mailToString.StartsWith(mailToPrefix, StringComparison.InvariantCultureIgnoreCase))
            {
                throw new InvalidObjectException("mailToString", null, mailToString);
            }

            mailToString = mailToString.Substring(mailToPrefix.Length);
            string toAddress = string.Empty;
            string additionalInfo = string.Empty;
            int speratorIndex = mailToString.IndexOf('?');
            toAddress = speratorIndex < 0 ? mailToString : mailToString.Substring(0, speratorIndex);
            mailMessage.To.Add(new MailAddress(toAddress));

            if (speratorIndex > 0 && speratorIndex < mailToString.Length - 1)
            {
                mailToString = mailToString.Substring(speratorIndex + 1);
            }

            WebDictionary webDictionary = WebUtil.Instance.ConvertKeyValuePairCollectionFromString(mailToString);
            mailMessage.Subject = webDictionary.GetHtmlDecodedValue("subject");
            mailMessage.Body = webDictionary.GetHtmlDecodedValue("body");

            var cc = ValidateMailAddress(webDictionary.GetValue("cc"));
            if (cc != null)
            {
                mailMessage.CC.Add(cc);
            }

            var bcc = ValidateMailAddress(webDictionary.GetValue("bcc"));
            if (bcc != null)
            {
                mailMessage.Bcc.Add(bcc);
            }

            return mailMessage;
        }

        #endregion

        /// <summary>
        /// Validates the mail address.
        /// </summary>
        /// <param name="mailAddress">The mail address.</param>
        /// <returns></returns>
        public static MailAddress ValidateMailAddress(string mailAddress)
        {
            return MailExtensions.ValidateMailAddress(mailAddress);
        }

        /// <summary>
        /// Sets the password.
        /// </summary>
        /// <param name="password">The password.</param>
        public void SetPassword(string password)
        {
            this.Password = password;
        }
    }
}
